@@ -7,6 +7,7 @@ blav_model_test <- function(lavmodel       = NULL,
                             lavcache       = NULL,
                             lavdata        = NULL,
                             lavjags        = NULL,
+                            samplls        = NULL,
                             jagextra       = NULL,
                             control        = list()) {
 
@@ -26,7 +27,8 @@ blav_model_test <- function(lavmodel       = NULL,
     }
 
     ppp <- postpred(lavpartable, lavmodel, lavoptions,
-                    lavsamplestats, lavdata, lavcache, lavjags)$ppval
+                    lavsamplestats, lavdata, lavcache, lavjags,
+                    samplls)$ppval
 
     TEST[[1]] <- list(test="mloglik",
                       stat=as.numeric(mll),
